import math

class laser_turtle:
    """ Implémentation d'un module inspiré de Turtle pour commande
    d'une machine à découpe et gravure laser """
    def __init__(self, width, height):
        """ Une image vectorielle est créée de taille width x height en millimètres
        Le curseur a une position x,y (point 0,0 en haut à gauche), et une orientation (heading)
        L'image contient 2 calques, un pour la découpe, un pour le dessin """
        self.cursor_x = 0
        self.cursor_y = 0
        self.cursor_heading = 90
        self.cursor_cut = False
        self.cursor_draw = True
        self.width = width
        self.height = height
        self.draw = ""
        self.cut = ""

    def moveto(self, x, y):
        """ Déplace le curseur à une position absolue (x,y), sans dessiner """
        self.cursor_x, self.cursor_y = x, y
        
    def heading(self, angle):
        """ Oriente le curseur à un angle absolu (0 pour Nord) Sens horaire. """
        self.cursor_heading = angle % 360

    def rotate(self, angle):
        """ Pivote le curseur d'un angle relatif (positif dans le sens horaire) """
        self.cursor_heading = (self.cursor_heading + angle) % 360
        
    def forward(self, d):
        """ Avance le curseur d'une distance d, en générant un chemin de type ligne """
        dx = d * math.cos ((self.cursor_heading - 90)*math.pi/180)
        dy = d * math.cos ((self.cursor_heading - 180)*math.pi/180)
        if self.cursor_draw :
            self.draw += "<path fill='none' stroke='black' stroke-width='0.2' d='M " + str(self.cursor_x) +"," + str(self.cursor_y) + " l " + str(dx) + " " + str(dy) + "'></path>\n"
        if self.cursor_cut :
            self.cut += "<path fill='none' stroke='red' stroke-width='0.2' d='M " + str(self.cursor_x) +"," + str(self.cursor_y) + " l " + str(dx) + " " + str(dy) + "'></path>\n"
        self.cursor_x = self.cursor_x + dx
        self.cursor_y = self.cursor_y + dy

    def backward(self,d):
        """ Recule le curseur d'une distance d, en générant un chemin de type ligne """
        self.forward(-d)
        
    def turn(self, r, a):
        """ Fait parcourir au curseur un arc de cercle de rayon r et d'angle a """
        tour = 1 if a >= 360 else -1 if a <= -360 else 0
        a = a % 360 if a >= 0 else a % 360 - 360
        alpha = self.cursor_heading + 90 if a >= 0 else self.cursor_heading - 90
        cx = self.cursor_x + r * math.sin(alpha*math.pi/180)
        cy = self.cursor_y - r * math.cos(alpha*math.pi/180)
        dx = cx - self.cursor_x + (self.cursor_x - cx)* math.cos(a*math.pi/180) - (self.cursor_y - cy)* math.sin(a*math.pi/180)
        dy = cy - self.cursor_y + (self.cursor_y - cy)* math.cos(a*math.pi/180) + (self.cursor_x - cx)* math.sin(a*math.pi/180)
        if tour != 0 :
            if self.cursor_draw :
                self.draw += "<circle fill='none' stroke='black' stroke-width='0.2' cx="+ str(cx) +" cy="+ str(cy) +" r="+ str(r) +"></circle>\n"
            if self.cursor_cut :
                self.draw += "<circle fill='none' stroke='red' stroke-width='0.2' cx="+ str(cx) +" cy="+ str(cy) +" r="+ str(r) +"></circle>\n"
        if self.cursor_draw :
            self.draw += "<path fill='none' stroke='black' stroke-width='0.2' d='M"+ str(self.cursor_x) +"," + str(self.cursor_y) + " a " + str(r) + " " + str(r) + " 0 " + (" 1 " if abs(a)>=180 else " 0 ") + (" 1 " if a>=0 else " 0 ") + str(dx) + " " + str(dy) + "'></path>\n"
        if self.cursor_cut :
            self.cut += "<path fill='none' stroke='red' stroke-width='0.2' d='M"+ str(self.cursor_x) +"," + str(self.cursor_y) + " a " + str(r) + " " + str(r) + " 0 " + (" 1 " if abs(a)>=180 else " 0 ") + (" 1 " if a>=0 else " 0 ") + str(dx) + " " + str(dy) + "'></path>\n"
        self.cursor_x = self.cursor_x + dx
        self.cursor_y = self.cursor_y + dy
        self.cursor_heading = self.cursor_heading + a

    def draw_on(self):
        """ Positionne le curseur en position de dessin """
        self.cursor_draw = True
        self.cursor_cut = False

    def draw_off(self):
        """ Remonte le curseur """
        self.cursor_cut = False
        self.cursor_draw = False

    def cut_on(self):
        """ Positionne le curseur en position de découpe """
        self.cursor_cut = True
        self.cursor_draw = False

    def cut_off(self):
        """ Remonte le curseur """
        self.cursor_cut = False
        self.cursor_draw = False

    def to_svg(self):
        """ Renvoie le dessin vectoriel en svg """
        s = "<svg width='"+str(self.width)+"mm' height='"+str(self.height)+"mm' viewBox='0 0 "+str(self.width)+" "+str(self.height)+"' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink' version='1.2' baseProfile='tiny'>\n"
        s += "<g id='draw'>\n"
        s += self.draw
        s += "</g>\n"
        s += "<g id='cut'>\n"
        s += self.cut
        s += "</g>\n"
        s += "</svg>\n"
        return s
        
    def export_svg(self, nom):
        """ Enregistre le dessin vectoriel en svg dans le fichier nom.svg """
        f = open(nom+".svg", mode="w", encoding="utf-8")
        f.write(self.to_svg())
        f.close()

    def _repr_svg_(self):
        """ Affichage pour Ipython et notebook jupyter """
        return self.to_svg()

""" Fin du module : laser_turtle """
